;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'mail '("sieve-manage"))
(custom-add-loads 'tools '("sieve"))
(custom-add-loads 'languages '("sieve"))
(custom-add-loads 'sieve '("sieve-mode" "sieve"))

;;; custom-load.el ends here
